/******************************************************
IAR +STM8S_StdPeriph_Lib_V2.1.0
Chip type               : STM8S003K3
MCU Core Clock frequency: 16 MHz
Project: HV sinewave inverter with HCPL3120&UC3842/43
*****************************************************/
#include "stm8s.h"
#include "iostm003.c"
#include <stdbool.h>
#include <math.h>


#define RHV_upr 680000UL //Ohms
#define RHV_lwr 10000   //Ohms
#define UHV_max 	((500UL*(RHV_upr+RHV_lwr))/RHV_lwr) //=U_max*100, 345V*100


enum  {start,work,loHV,ovlp} dev_st=start;

#define ok_led PD_ODR_ODR5
#define ovl_led PD_ODR_ODR6
#define loHV_led PD_ODR_ODR7

#define inv_off do{PB_ODR|=3;TIM1->BKR&=TIM1_BKR_MOE;}while(0)
#define inv_on do {PB_ODR&=~3;TIM1->BKR=TIM1_BKR_MOE;}while(0)

#define tnum 3
u16 tim[tnum];

bool adc_ready = 0;
u8 adc3val;

bool lo_hv_bit=0,restart_bit=1;

#define cpu_freq 16000000UL/2
#define cnt_res 0xff
#define pwm_freq  cpu_freq/(2*cnt_res)
#define base_freq_50 50 //Hz
#define s_step pwm_freq/(base_freq_50*2)
#define pwm_top 0xff
static u8 s_table [s_step/2];

void gpio_cfg()
{
  PA_ODR= 1<<1; //pullup for reset
  PB_ODR= 0;
  PC_ODR= 0;
  PD_ODR= 0xe0;
  PE_ODR= 0;
  PF_ODR= 0;

  PA_CR1= 0xff;
  PB_CR1= 0xff-12; //pb3_ADC3, ADC2 in
  PC_CR1= 0xff;
  PD_CR1= 0xff;
  PE_CR1= 0xff;
  PF_CR1= 0xff;

  PA_DDR= 1<<3|1<<2|1<<1;
  PB_DDR= 0xff-12;
  PC_DDR= 0xff;
  PD_DDR= 0xff;
  PE_DDR= 0xff;
  PF_DDR= 0;
}
void clk_cfg()
{
 CLK_DeInit();
 CLK->CKDIVR = 0;
 CLK->PCKENR1 = CLK_PCKENR1_TIM1|CLK_PCKENR1_TIM4;
}
void t1_cfg()
{
  {
  	TIM1->ARRH    = 0;
  	TIM1->ARRL    = 0xff;
	TIM1->PSCRL   = 1;
  	TIM1->CCR1H=TIM1->CCR2H=0;
  	TIM1->CCR1L=TIM1->CCR2L=0xff;
	TIM1->CR1 = TIM1_COUNTERMODE_CENTERALIGNED1|TIM1_CR1_URS;
	TIM1->CR2 = TIM1_CR2_CCPC;
	TIM1->RCR = 1;
  	TIM1->IER = TIM1_IER_UIE;
  }

  TIM1->CR2   |= TIM1_CR2_MMS;
  TIM1->CCR4L	= 0xfd;
  TIM1->CCMR4 	= TIM1_OCMODE_PWM1;

  TIM1->CCER1 = TIM1_CCER1_CC1E|TIM1_CCER1_CC1NE|TIM1_CCER1_CC1NP;
  TIM1->CCER1|= TIM1_CCER1_CC2E|TIM1_CCER1_CC2NE|TIM1_CCER1_CC2NP;

  TIM1->CCMR1 = TIM1_OCMODE_PWM1|TIM1_CCMR_OCxPE;
  TIM1->CCMR2|= 0x40|TIM1_CCMR_OCxPE;

  TIM1->EGR  |= TIM1_EGR_COMG;

  TIM1->DTR  = 28; // dt delay (*62ns)
  TIM1->CR1 |= TIM1_CR1_CEN;
  TIM1->BKR |= TIM1_LOCKLEVEL_2;

}

void t4_cfg()
{
  TIM4->ARR   = 250-1;
  TIM4->PSCR  = 7;      //16MHz/250/2exp7 -> 500 Hz interrupt
  TIM4->IER   = 1;
  TIM4->CR1   = 1;
}
void adc_cfg()
{
  ADC1->CR1  = ADC1_PRESSEL_FCPU_D18;
  ADC1->CR2  = ADC1_ALIGN_LEFT|ADC1_CR2_EXTTRIG;
  ADC1->CSR  = 3;
  ADC1->TDRL = 1<<3;

  ADC1->CR1 |= ADC1_CR1_ADON;
}
void iwdg_cfg()
{
IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
IWDG_SetPrescaler(IWDG_Prescaler_128);
IWDG_SetReload(0);    //2ms
IWDG_Enable();
IWDG_ReloadCounter();
}


void devcon();

INTERRUPT_HANDLER(TIM1_X3, 11) //15625 Hz
{
static u16 mst=0;
u16 tmp_table;

static u8 i=0;
static bool rck=0, dir;

static u16 adc_cps=0;
static u8 adc_cnt=0;


if(adc_cnt<0xff)
{
  adc_cnt++;
  adc_cps+= ADC1->DRH;
}
else
{
  adc3val = adc_cps>>8;
  adc_cnt = adc_cps = 0;
  adc_ready = 1;
}

if(restart_bit){restart_bit=0;mst=0;}

#define ADCub310 (((u32)31000*0xff)/UHV_max)  //229 adc points
static u8 uhv_tmp=0;

if(mst<0xff/2);
else if(ADC1->DRH>(u8)((20000UL*0xff)/UHV_max))
		{mst=((u16)ADCub310*0xff)/ADC1->DRH;uhv_tmp=lo_hv_bit=0;}
else 	{mst=0x1ff;if(uhv_tmp<0xff){uhv_tmp++;} else lo_hv_bit=1;}

tmp_table=(u16)(((u32)s_table [i]*mst)>>8);
if(tmp_table>0xff){tmp_table=0xff;}


 if(!i)
 {
 dir=1;i=1;
   if(rck)
   {
   rck=0;
  	TIM1->CCMR1 = TIM1_OCMODE_PWM1|TIM1_CCMR_OCxPE;
	TIM1->CCMR2 = 0x40;
   }
   else
   {
   rck=1;
	TIM1->CCMR1 = 0x40;
	TIM1->CCMR2 = TIM1_OCMODE_PWM1|TIM1_CCMR_OCxPE;
   }

	if(mst<0xff/2)mst+=5;
   TIM1->EGR|=TIM1_EGR_COMG;

 }
 else if((dir)&&(i<s_step/2-1))
 {
   TIM1->CCR1L=TIM1->CCR2L=(u8)tmp_table;
   i++;
 }
 else
 {
   dir=0; i--;
   TIM1->CCR1L=TIM1->CCR2L=(u8)tmp_table;
 }

  IWDG->KR = IWDG_KEY_REFRESH;
  TIM1->SR1 = 0;
}

INTERRUPT_HANDLER(TIM4_OVF, 23)     //500 Hz
{

 for(u8 i=0;i<tnum;i++)
	if(tim[i])tim[i]--;

 TIM4->SR1 = 0;
}



void main()
{
 gpio_cfg();
 clk_cfg();
 adc_cfg();
 ITC_SetSoftwarePriority(ITC_IRQ_TIM1_OVF, ITC_PRIORITYLEVEL_3);
 ITC_SetSoftwarePriority(ITC_IRQ_TIM4_OVF, ITC_PRIORITYLEVEL_1);
 asm("sim");
 t1_cfg();
 t4_cfg();


 for(u8 i=0;i<s_step/2;i++) s_table[i]=(u8)((float)pwm_top*sin(((3.1416/2)*i)/(s_step/2-1)));

 iwdg_cfg();
 asm("rim");
 PD_ODR=0;
w:
 {

	devcon();

 }
 goto w;

}

void devcon()//start,work,loHV,ovlp
{
  static bool itr=0;


 switch (dev_st)
 {
  case start:
  {
    if(!itr)
    {
		itr=1;
		inv_off;
		ok_led=ovl_led=loHV_led=0;
    }

	 if(adc_ready)
	 {
		adc_ready=0;
		if(adc3val>(u32)28000*0xff/UHV_max)
		{
		 dev_st=work;
		}
		else dev_st=loHV;
	 }


	 if(dev_st!=start){itr=0;}
	}
  break;

  case work:
  {
	 if(!itr)
    {
		itr=1;
		ok_led=1;
		restart_bit=1;
		inv_on;
		tim[0]=1000;
    }

	if(!tim[0])
	{
	 if(adc_ready)
	 {
		adc_ready=0;
		if(adc3val<(u8)((u32)26000*0xff/UHV_max))
		{
		 dev_st=loHV;
		}
	 }

	 static u8 lo_out_cnt=0;
	  if(!PB_IDR_IDR2&&!tim[1])
	  {
		tim[1]=5;
		if(lo_out_cnt<100)lo_out_cnt++;
		else {lo_out_cnt=0; dev_st=ovlp;}
	  }
	  else if (!tim[1]) if(lo_out_cnt)lo_out_cnt--;
	}

	if(dev_st!=work){itr=0;}
  }
  break;

  case loHV:
  {
	 if(!itr)
    {
		itr=1;
		inv_off;
		ok_led=ovl_led=0;loHV_led=1;
		tim[0]=2500;
    }

	 if(tim[0])
	 {
		if(!tim[1]){tim[1]=200; loHV_led^=1;}
	 }
	 else if(adc_ready)
	 {
		adc_ready=0;
		if(adc3val>(u8)((u32)30000*0xff/UHV_max))
		{
		 dev_st=start;
		}
		else tim[0]=2500;
	 }



	if(dev_st!=loHV){itr=0;}
  }
  break;

  case ovlp:
  {
	 if(!itr)
    {
		itr=1;
		inv_off;
		ok_led=0;ovl_led=1;loHV_led=0;
		tim[0]=5000;
    }

static u8 ovlp_cnt=0;

	 if(tim[0])
	 {
		if(!tim[1]){tim[1]=200; ovl_led^=1;}
	 }
	 else if(ovlp_cnt<0x0f) {dev_st=start;ovlp_cnt++;}
	 else ovl_led=1;



	if(dev_st!=ovlp){itr=0;}
  }
  break;

 default: break;
 }
}


